package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.util.RequestUtil;
import com.starbuds.server.api.notify.v1.facade.AppleFacade;
import com.starbuds.server.common.pojo.finance.PayAppleNotifyHisDo;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.PayAppleNotifyHisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.ArrayList;
import java.util.List;

/**
 * 苹果充值回调
 *
 * @author: ruomu
 * @date: 2021/11/2 created
 */

@Path("/v1/apple")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AppleFacadeImpl extends BaseFacade implements AppleFacade {

    @Reference(version = "1.0.0")
    PayAppleNotifyHisService payAppleNotifyHisService;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    /**
     * 苹果充值回调
     *
     * @param request
     * @return
     */
    @POST
    @Path("notify")
    @Consumes(ContentType.APPLICATION_JSON_UTF_8)
    @PermitAll
    @Override
    public String appleNotify(@Context HttpServletRequest request) {
        return exce(request);
    }

    /**
     * 苹果沙盒充值回调
     *
     * @param request
     * @return
     */
    @POST
    @Path("sandboxnotify")
    @Consumes(ContentType.APPLICATION_JSON_UTF_8)
    @PermitAll
    @Override
    public String sandboxNotify(@Context HttpServletRequest request) {
        return exce(request);
    }


    private String exce(HttpServletRequest request) {
        try {
            String notify = RequestUtil.httpServletRequestToString(request);
            log.info("appleinfo:" + notify);
            if (StringUtils.isEmpty(notify)) {
                return "fail";
            }

            JSONObject jsonObject = JSONObject.parseObject(notify);
            String notificationType = jsonObject.getString("notification_type");
            //退款
            if (!notificationType.equals("REFUND")) {
                return "success";
            }

            //插入总记录
            PayAppleNotifyHisDo payAppleNotifyHisDo = new PayAppleNotifyHisDo();
            Long time = System.currentTimeMillis();
            Long notifyId = identityService.getId();
            payAppleNotifyHisDo.setHisId(identityService.getId());
            payAppleNotifyHisDo.setNotifyId(notifyId);
            payAppleNotifyHisDo.setNotificationType(notificationType);
            payAppleNotifyHisDo.setResult(notify);
            payAppleNotifyHisDo.setCreateTime(time);
            payAppleNotifyHisService.insert(payAppleNotifyHisDo);


            //解析 unified_receipt
            JSONObject unifiedReceipt = jsonObject.getJSONObject("unified_receipt");

            if (unifiedReceipt != null) {
                JSONArray latestReceiptInfo = unifiedReceipt.getJSONArray("latest_receipt_info");
                List<PayAppleNotifyHisDo> items = new ArrayList<>();
                String environment = unifiedReceipt.getString("environment");
                if (latestReceiptInfo != null && latestReceiptInfo.size() > 0) {
                    for (int i = 0; i < latestReceiptInfo.size(); i++) {
                        JSONObject latestReceipt = latestReceiptInfo.getJSONObject(i);
                        PayAppleNotifyHisDo item = new PayAppleNotifyHisDo();
                        item.setHisId(identityService.getId());
                        item.setNotifyId(notifyId);
                        item.setNotificationType(notificationType);
                        item.setEnvironment(environment);
                        item.setTransactionId(latestReceipt.getString("transaction_id"));
                        item.setOriginalTransactionId(latestReceipt.getString("original_transaction_id"));
                        item.setResult(latestReceipt.toJSONString());
                        item.setCreateTime(time);
                        items.add(item);
                    }
                }
                if (items.size() > 0) {
                    payAppleNotifyHisService.insertItems(items);
                }
            }


        } catch (Exception e) {
            log.error("appleNotify", e);
            return "failerror";
        }

        return "success";
    }


}
