package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.v1.entity.form.SailtobCashoutNotifyForm;
import com.starbuds.server.api.notify.v1.facade.SailtobFacade;
import com.starbuds.server.common.constant.finance.ThirdPartyCallbackEnum;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisSaveBO;
import com.starbuds.server.common.pojo.finance.bo.ThirdPartyCallbackBO;
import com.starbuds.server.common.service.finance.SailtobService;
import com.starbuds.server.common.service.finance.ThirdPartyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.security.PermitAll;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.math.BigDecimal;
import java.util.Map;

/**
 * 智企行回调实现
 *
 * @author qinjiang
 * @date 2021/10/27
 */
@Slf4j
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Path("/v1/sailtob")
public class SailtobFacadeImpl extends BaseFacade implements SailtobFacade {

    public static final String SUCCESS = "success";
    public static final String FAIL = "fail";

    @Reference(version = "1.0.0")
    SailtobService sailtobService;

    @Reference(version = "1.0.0")
    ThirdPartyService thirdPartyService;

    /**
     * 提现回调通知
     *
     * @param jsonObject
     * @return
     */
    @Override
    @POST
    @Path("cashout")
    @Consumes(ContentType.APPLICATION_JSON_UTF_8)
    @PermitAll
    public String cashoutNotify(JSONObject jsonObject) {
        String receiveJSONString = jsonObject.toJSONString();
        log.info("智企行： jsonObject:" + jsonObject);
        String sign = jsonObject.getString("sign");
        jsonObject.remove("sign");

        Map<String, String> map = JSON.parseObject(jsonObject.toJSONString(), Map.class);
        map.put("amt", jsonObject.getBigDecimal("amt").toPlainString());

        boolean isVerify = sailtobService.verify(map, sign);
        if (isVerify) {
            log.info("智企行：解析签名错误");
            return FAIL;
        }

        SailtobCashoutNotifyForm cashoutNotifyForm = JSON.parseObject(receiveJSONString, SailtobCashoutNotifyForm.class);
        String state = cashoutNotifyForm.getState();
        if (StringUtils.isEmpty(state)) {
            log.info("智企行：返回订单状态为空");
            return FAIL;
        }

        if ("UNT".equals(state) || "PRO".equals(state) || "SETTING".equals(state) || "REFPRO".equals(state)) {
            log.info("智企行：订单状态处于进行中的状态：" + state);
            return SUCCESS;
        }

        ThirdPartyCallbackHisSaveBO his = new ThirdPartyCallbackHisSaveBO();
        his.setInfoType(ThirdPartyCallbackEnum.SAILTOB_CASHOUT_NOTIFY);
        his.setDetail(jsonObject.toJSONString());
        his.setKeyword(String.format("%s,%s,%s", cashoutNotifyForm.getLevyCode(), cashoutNotifyForm.getMerOrderId(), cashoutNotifyForm.getState()));
        boolean flag = thirdPartyService.saveCallbackHis(his);
        if (!flag) {
            log.error("智企行：提现回调通知 保存失败 {}", receiveJSONString);
            return FAIL;
        }

        // 验证数据
        Long userWalletWithdrawHisId = NumberUtils.toLong(cashoutNotifyForm.getMerOrderId());
        if (userWalletWithdrawHisId < 1) {
            log.warn("智企行：提现回调通知 贵方订单号 merOrderID 有误 {}", cashoutNotifyForm.getMerOrderId());
            return FAIL;
        }

//        Boolean isSuccess = false;
//        ThirdPartyWithdrawStatusEnum statusEnum = CALLBACK_FAIL;
//        switch (cashoutNotifyForm.getState()) {
//            case "SETSUCC": {
//                //成功
//                isSuccess = true;
//                statusEnum = CALLBACK_SUSSES;
//            } break;
//            case "SETFAIL": {
//                //失败
//                isSuccess = false;
//                statusEnum = CALLBACK_FAIL;
//            } break;
//            case "TICREF": {
//                //退票 (成功后退票由业务内部判断)
//                isSuccess = true;
//                statusEnum = CALLBACK_TICREF;
//            } break;
//            default: break;
//        }


        ThirdPartyCallbackBO callback = new ThirdPartyCallbackBO();
        callback.setUserWalletWithdrawHisId(userWalletWithdrawHisId);
        callback.setOutId(cashoutNotifyForm.getPayItemId());
        callback.setIsSuccess("SETSUCC".equals(cashoutNotifyForm.getState()));
//        callback.setWithdrawStatus(statusEnum);
        callback.setCallbackData(receiveJSONString);
        callback.setCallbackMsg(cashoutNotifyForm.getResMsg());

        return thirdPartyService.processWithdrawCallback(callback) ? SUCCESS : FAIL;

        //{"levyCode":"1000030412","splitFlag":"NOSPLIT","sign":"Mz8vYbDm2h9gD34mfpQe9xPL0oyEVj8QVbVCYi7ze6Pk5E4+LiJzMwfQnUJLOwjrwqmbBkHHVwLqVdfqm8Vanvmzv9OlFjr8n7+dOAbhmEFYRN0XeTzAxf9SrbN8k9fhRV28lRJrRRA7dJkOzH9+mQ44C587CAERhsF5E4ZBOvY=","amt":0.01,"resCode":"0000","merId":"6000010711","state":"SETSUCC","resMsg":"￦ﾈﾐ￥ﾊﾟ","payItemId":"1453261855718899712","reqId":"1635319576847","merOrderId":"1635319576847"}
    }
}
