package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.http.HttpUtil;
import com.starbuds.server.api.notify.v1.facade.UserAdFacade;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.UserAdHisDo;
import com.starbuds.server.common.pojo.mq.QueueUserFirstRechargeMsg;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.user.UserADHisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.*;

/**
 * 巨量广告回调地址
 *
 * @author: ruomu
 * @date: 2021/9/28 created
 */
@Path("/v1/ad")
@Component
@Slf4j
public class UserAdFacadeImpl extends BaseFacade implements UserAdFacade {

    @Reference(version = "1.0.0")
    UserADHisService userADHisService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    /**
     * 测试创建
     *
     * @return
     */
    @GET
    @Path("monitorTest")
    @PermitAll
    @Override
    public String monitorTest(@QueryParam("name") String name) {
        log.info("monitorTest:" + name);
        return name;
    }

    /**
     * 创建
     *
     * @return
     */
    @GET
    @Path("create")
    @PermitAll
    @Override
    public String monitor() {

        UserAdHisDo userAdHisDo = new UserAdHisDo();
        userAdHisDo.setIdfa("");
        userAdHisDo.setImei("");
        userAdHisDo.setImei("");
        userAdHisDo.setIp("");
        String queryString = getRequest().getQueryString();
        log.info("ad:create:::" + queryString);
        if (StringUtils.isEmpty(queryString)) {
            return "参数无效";
        }

        String osString = getRequest().getParameter("os");
        if (StringUtils.isEmpty(queryString) || !StringUtils.isNumeric(osString)) {
            return "os参数无效";
        }

        int os = 0;
        //传入的数据 0, "安卓"  1, "IOS"  3, 其他：
        //我们数据库 1, "Web"  2, "iOS"  3, "Android"
        if (StringUtils.isNotEmpty(osString)) {
            os = Integer.parseInt(osString);
            if (os == 0) {
                os = 3;
            } else if (os == 1) {
                os = 2;
            } else {
                os = 1;
            }
        }

        userAdHisDo.setAdId(identityService.getId());
        userAdHisDo.setOs(os);
        if (os == 2) {
            String idfa = getRequest().getParameter("idfa");
            if (StringUtils.isEmpty(idfa)) {
                return "idfa参数无效";
            }
            userAdHisDo.setIdfa(idfa);
        }


        if (os == 3) {
            String imei = getRequest().getParameter("imei");
            if (StringUtils.isEmpty(imei)) {
                imei = " ";
            }
            userAdHisDo.setImei(imei);
        }

        String ip = getRequest().getParameter("ip");
        if (StringUtils.isNotEmpty(ip)) {
            userAdHisDo.setIp(ip);
        }


        userAdHisDo.setParms(queryString);

        String callbackUrl = getRequest().getParameter("callback_url");
        if (StringUtils.isEmpty(callbackUrl)) {
            return "callback_url参数无效";
        }
        userAdHisDo.setCallbackUrl(getRequest().getParameter("callback_url"));
        userAdHisDo.setCreateTime(System.currentTimeMillis());
        userADHisService.insert(userAdHisDo);
        userAdHisDo.setCreateTime(System.currentTimeMillis());
        return "ok";
    }


}
